// This #include statement was automatically added by the Particle IDE.
#include <neopixel.h>
// This #include statement was automatically added by the Particle IDE.
#include <ArduinoJson.h>

#define ARDUINOJSON_ENABLE_ARDUINO_STRING 1

//setting up variables for lights
#define PIXEL_COUNT 17
#define PIXEL_PIN D2
#define PIXEL_TYPE WS2812B

//lights strip
Adafruit_NeoPixel strip(PIXEL_COUNT, PIXEL_PIN, PIXEL_TYPE);

//stores time.day to tell when the day is over + when to turn lights off
int currentDay;
//stores data retreived by sunset webhook
String sunset;
String sunsetHour;
String sunsetMinute;
//determine light behavior
bool on = false;
bool night = false;
bool lastOn = false;
bool triggeredToday = false;
bool gotColor = false;
//light strip variable so i can use it for for-loops
int pixelCount = 17;
//will store number of asteroids
int numAs;
//potentiometer
int analogPin = A5;

//date variables--end date has a default so that there are not Problems
String startDate;
String endDate = "2023-01-19";

//doc for sunset response
//StaticJsonDocument<512> doc;

//color of circling light
uint color;

void setup() {
    
    // Subscribe to the webhook response event
    Particle.subscribe("hook-response/get_sunset", myHandler);
    Particle.subscribe("hook-response/get_asteroids", singleValueHandler);
    Serial.begin(9600);
  
    //set time zone so that it turns off at midnight our time
    Time.zone(-6);

    //lights setup  
    strip.begin();

    //turn all lights off
    for (int i = 0; i < pixelCount; i++) {
        strip.setPixelColor(i, 0, 0, 0);
    }
    
    on = false;
    strip.show();
    
    currentDay = Time.day() - 1;
    
}

void loop() {
    
    //setting up time to compare sunset time to
    String currentHour = "0";
    int hour = Time.hourFormat12();
    hour += 6;
    if (hour > 12) {
        hour -= 12;
    }
    if (hour <= 0) {
        hour += 12;
    }
    String hourString = String(hour);
    currentHour.concat(hourString);
    
    String currentMin = "0";
    int minute = Time.minute();
    String minString = String(minute);
    currentMin.concat(minString);
    
    int val = analogRead(analogPin);
    on = (val > 2000);
    
    //using particle time functions, check if it is currently the currentDay
    if (Time.day() != currentDay) {
        //if it is not, get the sunset time
        triggeredToday = false;
        Particle.publish("get_sunset", PRIVATE);
        //and update the current day to the new day
        currentDay = Time.day();
        //and turn off the lights
        for (int i = 0; i < pixelCount; i++) {
            strip.setPixelColor(i, 0, 0, 0);
        }
        strip.show();
        
        on = false;
        night = false;
        
        //setting up date for url--going from yesterday to today
        startDate = endDate;
        
        String year = String(Time.year());
        endDate = year;
        endDate.concat("-");
        
        String month = String(Time.month());
        if (month.length() < 2) {
            month = "0" + month;
        }
        endDate.concat(month);
        endDate.concat("-");
        
        String day = String(Time.day());
        if (day.length() < 2) {
            day = "0" + day;
        }
        endDate.concat(day);
        
        lastOn = false;
    }
    
    sunsetHour = sunset.substring(0, 2);
    sunsetMinute = sunset.substring(3, 5);

    //using particle time functions, check if it is currently past today's sunset time
    if (currentHour.endsWith(sunsetHour) && currentMin.endsWith(sunsetMinute) && Time.isPM() && (!night) && triggeredToday) {
        //set up date to send to webhook
        String data = String::format("{ \"startDate\": \"%s\" } , { \"endDate\": \"%s\" }", startDate.c_str(), endDate.c_str());
        //get data. set up the number of colored lights required.
        Particle.publish("get_asteroids", data, PRIVATE);
        night = true;
        gotColor = false;
    }
    
    if (night && on && (!lastOn) && gotColor) {
        //set up the initial array and turn the lights on
        for (int i = 0; i < pixelCount; i++) {
            strip.setPixelColor(i, 0, 0, 2);
        }
        //uint color = strip.Color(10, 0, 10);
        strip.setPixelColor(0, color);
        strip.show();
        lastOn = true;
    }
    
    if (night && on && lastOn) {
        //if they are on, move the lights by one light
        uint temp1;
        uint temp2;
    
        for (int i = 0; i < pixelCount; i++) {
            temp2 = strip.getPixelColor(i);
            strip.setPixelColor(i, temp1);
            temp1 = temp2;
        }
    
        strip.setPixelColor(0, temp1);
        strip.show();
        delay(200);
        lastOn = true;
    }
    
    if (!on) {
        for (int i = 0; i < pixelCount; i++) {
            strip.setPixelColor(i, 0, 0, 0);
        }
        strip.show();
        lastOn = false;
    }
    
}


//sunset webhook handler
void myHandler(const char *event, const char *data) {
    sunset = String(data);
    triggeredToday = true;
}

//stupid webhook handler
void singleValueHandler(const char *event, const char *data) {
    
    String dataStr = String(data);
    numAs = atoi(dataStr);
    
    if (numAs > 100) {
        color = strip.Color(10, 0, 0);
    }
    else if (numAs > 75) {
        color = strip.Color(8, 1, 1);
    }
    else if (numAs > 50) {
        color = strip.Color(7, 2, 1);
    }
    else if (numAs > 25) {
        color = strip.Color(6, 2, 2);
    }
    else {
        color = strip.Color(5, 3, 2);
    }
    
    gotColor = true;
    
}




